package cn.nosum.http.proxy;

import cn.nosum.http.apache.InputStreamResponseHandler;
import cn.nosum.http.apache.Utf8ResponseHandler;
import cn.nosum.http.entity.DownloadResponse;
import cn.nosum.http.exception.HttpErrorException;
import jodd.http.HttpResponse;
import lombok.extern.slf4j.Slf4j;
import okhttp3.Response;
import org.apache.http.Header;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.entity.ContentType;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Objects;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * HttpResponse代理类，
 *
 * @author Young
 */
@Slf4j
public class HttpResponseProxy {
    private static final Pattern PATTERN = Pattern.compile(".*filename=\"(.*)\"");

    private CloseableHttpResponse apacheHttpResponse;
    private HttpResponse joddHttpResponse;
    private Response okHttpResponse;

    public HttpResponseProxy(CloseableHttpResponse apacheHttpResponse) {
        this.apacheHttpResponse = apacheHttpResponse;
    }

    public HttpResponseProxy(HttpResponse joddHttpResponse) {
        this.joddHttpResponse = joddHttpResponse;
    }

    public HttpResponseProxy(Response okHttpResponse) {
        this.okHttpResponse = okHttpResponse;
    }

    /**
     * 获取文件名称.
     * 由于对象只能由一个构造方法实现，三个response对象必定且只有一个不为空
     *
     * @return 文件名称
     */
    public String getFileName() throws HttpErrorException{
        if (this.apacheHttpResponse != null) {
            return this.getFileName(this.apacheHttpResponse);
        }
        if (this.joddHttpResponse != null) {
            return this.getFileName(this.joddHttpResponse);
        }
        if (this.okHttpResponse != null) {
            return this.getFileName(this.okHttpResponse);
        }
        return null;
    }

    /**
     * 获取下载文件响应流.
     *
     * @return 文件响应流
     * @throws IOException IO 异常
     */
    public OutputStream getOutputStream() throws IOException {
        if (this.apacheHttpResponse != null) {
            try (InputStream inputStream = InputStreamResponseHandler.INSTANCE.handleResponse(this.apacheHttpResponse)) {
                return transfer(inputStream);
            }
        }
        if (this.joddHttpResponse != null) {
            return transfer(joddHttpResponse.bodyBytes());
        }
        if (this.okHttpResponse != null) {
            return transfer(this.okHttpResponse.body().bytes());
        }
        return null;
    }

    /**
     * 获取响应类型.
     *
     * @return 响应类型
     * @throws HttpErrorException HTTP 异常
     */
    public String getContentType() throws HttpErrorException, IOException {
        String contentType = null;

        // application/json; encoding=utf-8 下载媒体文件出错
        final String mimeType = ContentType.APPLICATION_JSON.getMimeType();

        if (this.apacheHttpResponse != null) {
            Header[] contentTypeHeader = apacheHttpResponse.getHeaders("Content-Type");
            if (contentTypeHeader[0].getValue().startsWith(mimeType)) {
                throw new HttpErrorException(Utf8ResponseHandler.INSTANCE.handleResponse(this.apacheHttpResponse));
            }
            contentType = contentTypeHeader[0].getValue();
        }

        if (this.joddHttpResponse != null) {
            contentType = this.joddHttpResponse.header("Content-Type");
            if (contentType != null && contentType.startsWith(mimeType)) {
                throw new HttpErrorException(this.joddHttpResponse.bodyText());
            }
        }

        if (this.okHttpResponse != null) {
            contentType = this.okHttpResponse.header("Content-Type");
            if (contentType != null && contentType.startsWith(mimeType)) {
                throw new HttpErrorException(Objects.requireNonNull(this.okHttpResponse.body()).string());
            }
        }
        return contentType;
    }


    /**
     * 转换响应流为下载文件响应.
     *
     * @return 下载文件响应
     * @throws HttpErrorException HTTP 异常
     * @throws IOException        IO 异常
     */
    public DownloadResponse transferDownload() throws HttpErrorException, IOException {
        return DownloadResponse.builder().contentType(getContentType()).outputStream(getOutputStream()).fileName(getFileName()).build();
    }

    private OutputStream transfer(byte[] bytes) throws IOException {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        byteArrayOutputStream.write(bytes);
        return byteArrayOutputStream;
    }

    private OutputStream transfer(InputStream inputStream) throws IOException {
        OutputStream output = new ByteArrayOutputStream();
        byte[] buffer = new byte[4096];
        int n;
        while (-1 != (n = inputStream.read(buffer))) {
            output.write(buffer, 0, n);
        }
        return output;
    }

    /**
     * 获取文件名称
     *
     * @param response apache 文件响应
     * @return 文件名称
     */
    private String getFileName(CloseableHttpResponse response) throws HttpErrorException {
        Header[] contentDispositionHeader = response.getHeaders("Content-disposition");
        if (contentDispositionHeader == null || contentDispositionHeader.length == 0) {
            log.info("无法获取到文件名称");
            return null;
        }
        return this.extractFileNameFromContentString(contentDispositionHeader[0].getValue());
    }

    /**
     * 获取文件名称
     *
     * @param response jodd 文件响应
     * @return 文件名称
     */
    private String getFileName(HttpResponse response) throws HttpErrorException {
        String content = response.header("Content-disposition");
        return this.extractFileNameFromContentString(content);
    }


    /**
     * 获取文件名称
     *
     * @param response okhttp3 文件响应
     * @return 文件名称
     */
    private String getFileName(Response response) throws HttpErrorException {
        String content = response.header("Content-disposition");
        return this.extractFileNameFromContentString(content);
    }

    /**
     * 文件名称校验.
     *
     * @param content 包含文件名称的请求头内容
     * @return 文件名称
     */
    private String extractFileNameFromContentString(String content) throws HttpErrorException {
        if (content == null || content.length() == 0) {
            log.info("file name is empty,check file stream!!");
            return null;
        }

        Matcher m = PATTERN.matcher(content);
        if (m.matches()) {
            return m.group(1);
        }
        throw new HttpErrorException("无法获取到文件名，header信息有问题");
    }

}
