package cn.nosum.wx.cp.api;

import cn.nosum.http.entity.DownloadResponse;
import cn.nosum.http.entity.UploadRequest;
import cn.nosum.wx.common.api.WxConsts;
import cn.nosum.wx.common.entity.result.WxMediaUploadResult;
import cn.nosum.wx.common.error.WxErrorException;

/**
 * 媒体管理接口.
 *
 * @author Young
 */
public interface WxCpMediaService {

    /**
     * 通过 URL 获取文件上传请求对象.
     *
     * @param url 文件地址
     * @return 文件上传请求对象
     * @throws WxErrorException 企业微信接口调用异常
     */
    UploadRequest getUploadRequest(String url) throws WxErrorException;

    /**
     * <pre>
     * 上传图片.
     * 上传图片得到图片URL，该URL永久有效
     * 返回的图片URL，仅能用于图文消息（mpnews）正文中的图片展示；若用于非企业微信域名下的页面，图片将被屏蔽。
     * 每个企业每天最多可上传100张图片
     * 接口url格式：https://qyapi.weixin.qq.com/cgi-bin/media/uploadimg?access_token=ACCESS_TOKEN
     * </pre>
     *
     * @param uploadRequest 上传文件对象封装
     * @return 返回图片url
     * @throws WxErrorException 企业微信接口调用异常
     */
    String uploadImg(UploadRequest uploadRequest) throws WxErrorException;

    /**
     * <pre>
     * 上传多媒体文件.
     * 上传的多媒体文件有格式和大小限制，如下：
     *   图片（image）: 1M，支持JPG格式
     *   语音（voice）：2M，播放长度不超过60s，支持AMR\MP3格式
     *   视频（video）：10MB，支持MP4格式
     *   缩略图（thumb）：64KB，支持JPG格式
     * 详情请见: http://mp.weixin.qq.com/wiki/index.php?title=上传下载多媒体文件
     * </pre>
     *
     * @param mediaType   媒体类型, 请看{@link WxConsts.MediaFileType}
     * @param inputStream 输入流，需要调用方控制关闭该输入流
     * @return 文件上传返回
     * @throws WxErrorException 企业微信接口调用异常
     */
    WxMediaUploadResult upload(String mediaType, UploadRequest inputStream) throws WxErrorException;

    /**
     * <pre>
     * 下载多媒体文件.
     * 根据微信文档，视频文件下载不了，会返回null
     * 详情请见: http://mp.weixin.qq.com/wiki/index.php?title=上传下载多媒体文件
     * </pre>
     *
     * @param mediaId 媒体id
     * @return 文件流
     * @throws WxErrorException 企业微信接口调用异常
     */
    DownloadResponse download(String mediaId) throws WxErrorException;

    /**
     * <pre>
     * 获取高清语音素材.
     * 可以使用本接口获取从JSSDK的uploadVoice接口上传的临时语音素材，格式为speex，16K采样率。该音频比上文的临时素材获取接口（格式为amr，8K采样率）更加清晰，适合用作语音识别等对音质要求较高的业务。
     * 请求方式：GET（HTTPS）
     * 请求地址：https://qyapi.weixin.qq.com/cgi-bin/media/get/jssdk?access_token=ACCESS_TOKEN&media_id=MEDIA_ID
     * 仅企业微信2.4及以上版本支持。
     * 文档地址：https://work.weixin.qq.com/api/doc#90000/90135/90255
     * </pre>
     *
     * @param mediaId 媒体id
     * @return 保存到本地的临时文件
     * @throws WxErrorException 企业微信接口调用异常
     */
    DownloadResponse getJsSdk(String mediaId) throws WxErrorException;


}
