package cn.nosum.wx.cp.api.impl;

import cn.nosum.http.RequestExecutor;
import cn.nosum.http.RequestHttp;
import cn.nosum.http.SimpleGetRequestExecutor;
import cn.nosum.http.SimplePostRequestExecutor;
import cn.nosum.http.exception.HttpError;
import cn.nosum.http.exception.HttpErrorException;
import cn.nosum.wx.common.api.WxConsts;
import cn.nosum.wx.common.entity.ToJson;
import cn.nosum.wx.common.entity.WxAccessToken;
import cn.nosum.wx.common.entity.WxJsapiSignature;
import cn.nosum.wx.common.error.WxError;
import cn.nosum.wx.common.error.WxErrorException;
import cn.nosum.wx.common.error.WxRuntimeException;
import cn.nosum.wx.common.utils.DataUtils;
import cn.nosum.wx.common.utils.RandomUtils;
import cn.nosum.wx.common.utils.crypto.SHA1;
import cn.nosum.wx.common.utils.json.GsonParser;
import cn.nosum.wx.cp.api.*;
import cn.nosum.wx.cp.config.WxCpConfigStorage;
import cn.nosum.wx.cp.entity.WxCpMaJsCode2SessionResult;
import com.google.common.base.Joiner;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.function.Consumer;
import java.util.function.Supplier;

import static cn.nosum.wx.cp.constant.WxCpApiPathConsts.*;

/**
 * 企业微信API服务层基础实现.
 *
 * @author Young
 */
@Slf4j
@Data
public abstract class BaseWxCpServiceImpl<H, P> implements WxCpService, RequestHttp<H, P> {

    private WxCpUserService userService = new WxCpUserServiceImpl(this);
    private WxCpChatService chatService = new WxCpChatServiceImpl(this);
    private WxCpDepartmentService departmentService = new WxCpDepartmentServiceImpl(this);
    private WxCpMediaService mediaService = new WxCpMediaServiceImpl(this);
    private WxCpMenuService menuService = new WxCpMenuServiceImpl(this);
    private WxCpOAuth2Service oauth2Service = new WxCpOAuth2ServiceImpl(this);
    private WxCpTagService tagService = new WxCpTagServiceImpl(this);
    private WxCpAgentService agentService = new WxCpAgentServiceImpl(this);
    private WxCpOaService oaService = new WxCpOaServiceImpl(this);
    private WxCpTaskCardService taskCardService = new WxCpTaskCardServiceImpl(this);
    private WxCpExternalContactService externalContactService = new WxCpExternalContactServiceImpl(this);
    private WxCpGroupRobotService groupRobotService = new WxCpGroupRobotServiceImpl(this);
    private WxCpMessageService messageService = new WxCpMessageServiceImpl(this);
    private WxCpOaCalendarService oaCalendarService = new WxCpOaCalendarServiceImpl(this);
    private WxCpOaScheduleService oaScheduleService = new WxCpOaOaScheduleServiceImpl(this);
    private WxCpAgentWorkBenchService workBenchService = new WxCpAgentWorkBenchServiceImpl(this);

    protected WxCpConfigStorage configStorage;

    private int retrySleepMillis = 1000;
    private int maxRetryTimes = 5;

    @Override
    public boolean checkSignature(String msgSignature, String timestamp, String nonce, String data) {
        try {
            return SHA1.gen(this.configStorage.getToken(), timestamp, nonce, data).equals(msgSignature);
        } catch (Exception e) {
            log.error("Checking signature failed, and the reason is :" + e.getMessage());
            return false;
        }
    }

    @Override
    public String getAccessToken() throws WxErrorException {
        return getAccessToken(false);
    }

    @Override
    public String getAccessToken(boolean forceRefresh) throws WxErrorException {
        String url = String.format(configStorage.getApiUrl(GET_TOKEN), this.configStorage.getCorpId(), this.configStorage.getCorpSecret());
        return getCacheWithUrl(this.configStorage.getAccessTokenLock(), url,
                configStorage::getAccessToken,
                () -> configStorage.isAccessTokenExpired() || forceRefresh,
                result -> {
                    WxAccessToken token = WxAccessToken.fromJson(result);
                    configStorage.updateAccessToken(token.getAccessToken(), token.getExpiresIn());
                });
    }

    @Override
    public String getAgentJsapiTicket() throws WxErrorException {
        return this.getAgentJsapiTicket(false);
    }

    @Override
    public String getAgentJsapiTicket(boolean forceRefresh) throws WxErrorException {
        String url = this.configStorage.getApiUrl(GET_AGENT_CONFIG_TICKET);
        return getCacheWithUrl(this.configStorage.getAgentJsapiTicketLock(), url,
                configStorage::getAgentJsapiTicket,
                () -> configStorage.isAgentJsapiTicketExpired() || forceRefresh,
                result -> {
                    JsonObject jsonObject = GsonParser.parse(result);
                    this.configStorage.updateAgentJsapiTicket(jsonObject.get("ticket").getAsString(), jsonObject.get("expires_in").getAsInt());
                });
    }

    @Override
    public String getJsapiTicket() throws WxErrorException {
        return getJsapiTicket(false);
    }

    @Override
    public String getJsapiTicket(boolean forceRefresh) throws WxErrorException {
        String url = this.configStorage.getApiUrl(GET_JSAPI_TICKET);
        return getCacheWithUrl(this.configStorage.getJsapiTicketLock(), url,
                configStorage::getJsapiTicket,
                () -> configStorage.isJsapiTicketExpired() || forceRefresh,
                result -> {
                    JsonObject ticketObj = GsonParser.parse(result);
                    this.configStorage.updateJsapiTicket(ticketObj.get("ticket").getAsString(), ticketObj.get("expires_in").getAsInt());
                });
    }

    /**
     * 通过URL获取内容,返回缓存后的内容.
     *
     * @param lock            获取缓存锁
     * @param url             URL
     * @param cacheSupplier   缓存提供者
     * @param refreshSupplier 是否刷新提供者
     * @param refreshConsumer 刷新消费者，用于存储到缓存当中
     * @return 缓存后的内容
     * @throws WxErrorException 企业微信接口调用异常
     */
    private String getCacheWithUrl(Lock lock, String url,
                                   Supplier<String> cacheSupplier,
                                   Supplier<Boolean> refreshSupplier,
                                   Consumer<String> refreshConsumer) throws WxErrorException {
        if (refreshSupplier.get()) {
            lock.lock();
            try {
                if (refreshSupplier.get()) {
                    String resultContent = this.executeNormal(SimplePostRequestExecutor.create(this), url, null);
                    WxError error = WxError.fromJson(resultContent);
                    if (error.getErrorCode() != 0) {
                        throw new WxErrorException(error);
                    }
                    refreshConsumer.accept(resultContent);
                }
            } finally {
                lock.unlock();
            }
        }
        return cacheSupplier.get();
    }

    @Override
    public WxJsapiSignature createJsapiSignature(String url) throws WxErrorException {
        long timestamp = System.currentTimeMillis() / 1000;
        String noncestr = RandomUtils.getRandomStr();
        String jsapiTicket = getJsapiTicket(false);
        String signature = SHA1.genWithAmple(
                "jsapi_ticket=" + jsapiTicket,
                "noncestr=" + noncestr,
                "timestamp=" + timestamp,
                "url=" + url
        );
        WxJsapiSignature jsapiSignature = new WxJsapiSignature();
        jsapiSignature.setTimestamp(timestamp);
        jsapiSignature.setNonceStr(noncestr);
        jsapiSignature.setUrl(url);
        jsapiSignature.setSignature(signature);
        jsapiSignature.setAppId(this.configStorage.getCorpId());
        return jsapiSignature;
    }

    @Override
    public WxCpMaJsCode2SessionResult jsCode2Session(String jsCode) {
        Map<String, String> params = new HashMap<>(2);
        params.put("js_code", jsCode);
        params.put("grant_type", "authorization_code");

        final String url = this.configStorage.getApiUrl(JS_CODE_TO_SESSION);
        return WxCpMaJsCode2SessionResult.fromJson(this.get(url, Joiner.on("&").withKeyValueSeparator("=").join(params)));
    }

    @Override
    public String[] getCallbackIp() {
        String responseContent = get(this.configStorage.getApiUrl(GET_CALLBACK_IP), null);
        JsonObject tmpJsonObject = GsonParser.parse(responseContent);
        JsonArray jsonArray = tmpJsonObject.get("ip_list").getAsJsonArray();
        String[] ips = new String[jsonArray.size()];
        for (int i = 0; i < jsonArray.size(); i++) {
            ips[i] = jsonArray.get(i).getAsString();
        }
        return ips;
    }

    @Override
    public String replaceParty(String mediaId) {
        JsonObject jsonObject = new JsonObject();
        jsonObject.addProperty("media_id", mediaId);
        return post(this.configStorage.getApiUrl(BATCH_REPLACE_PARTY), jsonObject.toString());
    }

    @Override
    public String replaceUser(String mediaId) {
        JsonObject jsonObject = new JsonObject();
        jsonObject.addProperty("media_id", mediaId);
        return post(this.configStorage.getApiUrl(BATCH_REPLACE_USER), jsonObject.toString());
    }

    @Override
    public String getTaskResult(String joinId) {
        String url = this.configStorage.getApiUrl(BATCH_GET_RESULT + joinId);
        return get(url, null);
    }

    @Override
    public String get(String url, String queryParam) {
        return execute(SimpleGetRequestExecutor.create(this), url, queryParam);
    }

    @Override
    public String post(String url, String postData) {
        return execute(SimplePostRequestExecutor.create(this), url, postData);
    }

    @Override
    public String post(String url, JsonObject jsonObject) {
        return this.post(url, jsonObject.toString());
    }

    @Override
    public String post(String url, ToJson obj) {
        return this.post(url, obj.toJson());
    }

    @Override
    public String post(String url, Object obj) {
        return this.post(url, obj.toString());
    }

    @Override
    public String postWithoutToken(String url, String postData) throws WxErrorException {
        return this.executeNormal(SimplePostRequestExecutor.create(this), url, postData);
    }

    /**
     * 向微信端发送请求，在这里执行的策略是当发生access_token过期时才去刷新，然后重新执行请求，而不是全局定时请求.
     */
    @Override
    public <T, R> R execute(RequestExecutor<T, R> executor, String uri, T data) {
        int retryTimes = 0;
        do {
            try {
                return this.executeInternal(executor, uri, data, false);
            } catch (WxErrorException e) {
                if (retryTimes + 1 > this.maxRetryTimes) {
                    log.warn("重试达到最大次数【{}】", this.maxRetryTimes);
                    throw new WxRuntimeException("微信服务端异常，超出重试次数");
                }

                HttpError error = e.getError();
                /*
                 * -1 系统繁忙, 1000ms后重试
                 */
                if (error.getErrorCode() == -1) {
                    int sleepMillis = this.retrySleepMillis * (1 << retryTimes);
                    try {
                        log.debug("微信系统繁忙，{} ms 后重试(第{}次)", sleepMillis, retryTimes + 1);
                        TimeUnit.MILLISECONDS.sleep(sleepMillis);
                    } catch (InterruptedException interruptedException) {
                        Thread.currentThread().interrupt();
                    }
                }
            }
        } while (retryTimes++ < this.maxRetryTimes);

        throw new WxRuntimeException("微信服务端异常，超出重试次数");
    }

    protected <T, R> R executeInternal(RequestExecutor<T, R> executor, String uri, T data, boolean doNotAutoRefresh) throws WxErrorException {
        T dataForLog = DataUtils.handleDataWithSecret(data);

        if (uri.contains("access_token=")) {
            throw new IllegalArgumentException("uri参数中不允许有access_token: " + uri);
        }
        String accessToken = getAccessToken(false);
        String uriWithAccessToken = uri + (uri.contains("?") ? "&" : "?") + "access_token=" + accessToken;

        try {
            R result = executor.execute(uriWithAccessToken, data);
            log.debug("\n【请求地址】: {}\n【请求参数】：{}\n【响应数据】：{}", uriWithAccessToken, dataForLog, result);
            return result;
        } catch (HttpErrorException e) {
            HttpError error = e.getError();

            if (WxConsts.ACCESS_TOKEN_ERROR_CODES.contains(error.getErrorCode())) {
                // 强制设置wxCpConfigStorage它的access token过期了，这样在下一次请求里就会刷新access token
                this.configStorage.expireAccessToken();
                if (this.getWxCpConfigStorage().autoRefreshToken() && !doNotAutoRefresh) {
                    log.warn("即将重新获取新的access_token，错误代码：{}，错误信息：{}", error.getErrorCode(), error.getErrorMsg());
                    //下一次不再自动重试
                    //当小程序误调用第三方平台专属接口时,第三方无法使用小程序的access token,如果可以继续自动获取token会导致无限循环重试,直到栈溢出
                    return this.executeInternal(executor, uri, data, true);
                }
            }

            if (error.getErrorCode() != 0) {
                log.error("\n【请求地址】: {}\n【请求参数】：{}\n【错误信息】：{}", uriWithAccessToken, dataForLog, error);
                throw new WxErrorException(error, e);
            }
            return null;
        } catch (IOException e) {
            log.error("\n【请求地址】: {}\n【请求参数】：{}\n【异常信息】：{}", uriWithAccessToken, dataForLog, e.getMessage());
            throw new WxRuntimeException(e);
        }
    }

    /**
     * 普通请求，不自动带accessToken
     */
    private <T, R> R executeNormal(RequestExecutor<T, R> executor, String uri, T data) throws WxErrorException {
        try {
            R result = executor.execute(uri, data);
            log.debug("\n【请求地址】: {}\n【请求参数】：{}\n【响应数据】：{}", uri, data, result);
            return result;
        } catch (HttpErrorException e) {
            HttpError error = e.getError();
            if (error.getErrorCode() != 0) {
                log.error("\n【请求地址】: {}\n【请求参数】：{}\n【错误信息】：{}", uri, data, error);
                throw new WxErrorException(error, e);
            }
            return null;
        } catch (IOException e) {
            log.error("\n【请求地址】: {}\n【请求参数】：{}\n【异常信息】：{}", uri, data, e.getMessage());
            throw new WxErrorException(e);
        }
    }

    @Override
    public void setWxCpConfigStorage(WxCpConfigStorage wxConfigProvider) {
        this.configStorage = wxConfigProvider;
        this.initHttp();
    }

    @Override
    public void setRetrySleepMillis(int retrySleepMillis) {
        this.retrySleepMillis = retrySleepMillis;
    }

    @Override
    public void setMaxRetryTimes(int maxRetryTimes) {
        this.maxRetryTimes = maxRetryTimes;
    }

    @Override
    public RequestHttp<?, ?> getRequestHttp() {
        return this;
    }

}
