package cn.nosum.wx.cp.config.impl;

import cn.nosum.http.RequestHttpProperties;
import cn.nosum.wx.common.entity.WxAccessToken;
import cn.nosum.wx.cp.config.WxCpConfigStorage;
import cn.nosum.wx.cp.constant.WxCpApiPathConsts;
import cn.nosum.wx.cp.utils.json.WxCpGsonBuilder;

import java.io.Serializable;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * 基于内存的企业微信配置.
 *
 * @author Young
 */
public class WxCpDefaultConfigImpl implements WxCpConfigStorage, Serializable {
    private static final long serialVersionUID = 1154541446729462780L;

    /**
     * The Access token.
     */
    protected volatile String accessToken;

    /**
     * The Access token lock.
     */
    protected transient Lock accessTokenLock = new ReentrantLock();

    /**
     * The Agent id.
     */
    protected volatile Integer agentId;

    /**
     * The Agent code.
     */
    protected volatile String agentCode;

    /**
     * The Jsapi ticket lock.
     */
    protected transient Lock jsapiTicketLock = new ReentrantLock();

    /**
     * The Agent jsapi ticket lock.
     */
    protected transient Lock agentJsapiTicketLock = new ReentrantLock();


    /**
     * The Http Properties.
     */
    private final RequestHttpProperties httpProperties = new RequestHttpProperties();

    private volatile String corpId;
    private volatile String corpSecret;
    private volatile String token;
    private volatile String aesKey;
    private volatile long expiresTime;
    private volatile String oauth2redirectUri;
    private volatile String jsapiTicket;
    private volatile long jsapiTicketExpiresTime;
    private volatile String agentJsapiTicket;
    private volatile long agentJsapiTicketExpiresTime;
    private volatile String baseApiUrl;
    private volatile String webhookKey;

    @Override
    public void setBaseApiUrl(String baseUrl) {
        this.baseApiUrl = baseUrl;
    }

    @Override
    public String getApiUrl(String path) {
        if (baseApiUrl == null) {
            baseApiUrl = WxCpApiPathConsts.DEFAULT_CP_BASE_URL;
        }
        return baseApiUrl + path;
    }

    @Override
    public String getAccessToken() {
        return this.accessToken;
    }

    /**
     * Sets access token.
     *
     * @param accessToken the access token
     */
    public void setAccessToken(String accessToken) {
        this.accessToken = accessToken;
    }

    @Override
    public Lock getAccessTokenLock() {
        return this.accessTokenLock;
    }

    @Override
    public boolean isAccessTokenExpired() {
        return System.currentTimeMillis() > this.expiresTime;
    }

    @Override
    public void expireAccessToken() {
        this.expiresTime = 0;
    }

    @Override
    public synchronized void updateAccessToken(WxAccessToken accessToken) {
        updateAccessToken(accessToken.getAccessToken(), accessToken.getExpiresIn());
    }

    @Override
    public synchronized void updateAccessToken(String accessToken, int expiresInSeconds) {
        this.accessToken = accessToken;
        this.expiresTime = System.currentTimeMillis() + (expiresInSeconds - 200) * 1000L;
    }

    @Override
    public String getJsapiTicket() {
        return this.jsapiTicket;
    }

    /**
     * Sets jsapi ticket.
     *
     * @param jsapiTicket the jsapi ticket
     */
    public void setJsapiTicket(String jsapiTicket) {
        this.jsapiTicket = jsapiTicket;
    }

    @Override
    public Lock getJsapiTicketLock() {
        return this.jsapiTicketLock;
    }

    /**
     * Gets jsapi ticket expires time.
     *
     * @return the jsapi ticket expires time
     */
    public long getJsapiTicketExpiresTime() {
        return this.jsapiTicketExpiresTime;
    }

    /**
     * Sets jsapi ticket expires time.
     *
     * @param jsapiTicketExpiresTime the jsapi ticket expires time
     */
    public void setJsapiTicketExpiresTime(long jsapiTicketExpiresTime) {
        this.jsapiTicketExpiresTime = jsapiTicketExpiresTime;
    }

    @Override
    public boolean isJsapiTicketExpired() {
        return System.currentTimeMillis() > this.jsapiTicketExpiresTime;
    }

    @Override
    public synchronized void updateJsapiTicket(String jsapiTicket, int expiresInSeconds) {
        this.jsapiTicket = jsapiTicket;
        // 预留200秒的时间
        this.jsapiTicketExpiresTime = System.currentTimeMillis() + (expiresInSeconds - 200) * 1000L;
    }

    @Override
    public String getAgentJsapiTicket() {
        return this.agentJsapiTicket;
    }

    @Override
    public Lock getAgentJsapiTicketLock() {
        return this.agentJsapiTicketLock;
    }

    @Override
    public boolean isAgentJsapiTicketExpired() {
        return System.currentTimeMillis() > this.agentJsapiTicketExpiresTime;
    }

    @Override
    public void expireAgentJsapiTicket() {
        this.agentJsapiTicketExpiresTime = 0;
    }

    @Override
    public void updateAgentJsapiTicket(String jsapiTicket, int expiresInSeconds) {
        this.agentJsapiTicket = jsapiTicket;
        // 预留200秒的时间
        this.agentJsapiTicketExpiresTime = System.currentTimeMillis() + (expiresInSeconds - 200) * 1000L;
    }

    @Override
    public void expireJsapiTicket() {
        this.jsapiTicketExpiresTime = 0;
    }

    @Override
    public String getCorpId() {
        return this.corpId;
    }

    /**
     * Sets corp id.
     *
     * @param corpId the corp id
     */
    public void setCorpId(String corpId) {
        this.corpId = corpId;
    }

    @Override
    public String getCorpSecret() {
        return this.corpSecret;
    }

    /**
     * Sets corp secret.
     *
     * @param corpSecret the corp secret
     */
    public void setCorpSecret(String corpSecret) {
        this.corpSecret = corpSecret;
    }

    @Override
    public String getToken() {
        return this.token;
    }

    /**
     * Sets token.
     *
     * @param token the token
     */
    public void setToken(String token) {
        this.token = token;
    }

    @Override
    public long getExpiresTime() {
        return this.expiresTime;
    }

    /**
     * Sets expires time.
     *
     * @param expiresTime the expires time
     */
    public void setExpiresTime(long expiresTime) {
        this.expiresTime = expiresTime;
    }

    @Override
    public String getAesKey() {
        return this.aesKey;
    }

    /**
     * Sets aes key.
     *
     * @param aesKey the aes key
     */
    public void setAesKey(String aesKey) {
        this.aesKey = aesKey;
    }

    @Override
    public Integer getAgentId() {
        return this.agentId;
    }

    /**
     * Sets agent id.
     *
     * @param agentId the agent id
     */
    public void setAgentId(Integer agentId) {
        this.agentId = agentId;
    }

    @Override
    public String getAgentCode() {
        return this.agentCode;
    }

    /**
     * Sets agent code.
     *
     * @param agentCode the agent code
     */
    public void setAgentCode(String agentCode) {
        this.agentCode = agentCode;
    }

    @Override
    public String getOauth2redirectUri() {
        return this.oauth2redirectUri;
    }

    /**
     * Sets oauth 2 redirect uri.
     *
     * @param oauth2redirectUri the oauth 2 redirect uri
     */
    public void setOauth2redirectUri(String oauth2redirectUri) {
        this.oauth2redirectUri = oauth2redirectUri;
    }

    @Override
    public boolean autoRefreshToken() {
        return true;
    }

    @Override
    public String getWebhookKey() {
        return this.webhookKey;
    }

    @Override
    public RequestHttpProperties getHttpProperties() {
        return this.httpProperties;
    }

    /**
     * Sets webhook key.
     *
     * @param webhookKey the webhook key
     * @return the webhook key
     */
    public WxCpDefaultConfigImpl setWebhookKey(String webhookKey) {
        this.webhookKey = webhookKey;
        return this;
    }

    @Override
    public String toString() {
        return WxCpGsonBuilder.create().toJson(this);
    }
}
